"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useState = useState;
exports.useEffect = useEffect;
exports.default = withHooks;

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var currentInstance = null;
var isMounting = false;
var callIndex = 0;

function useState(initial) {
  var id = ++callIndex;
  var state = currentInstance.state;
  var setState = currentInstance.setState.bind(currentInstance);

  var updater = function updater(newValue) {
    setState(_defineProperty({}, id, newValue));
  };

  if (isMounting) {
    state[id] = initial;
    return [initial, updater];
  } else {
    return [state[id], updater];
  }
}

function useEffect(rawEffect, deps) {
  var id = ++callIndex;

  if (isMounting) {
    var injectedCleanup = function injectedCleanup() {
      var current = injectedCleanup.current;

      if (current) {
        current();
        injectedCleanup.current = null;
      }
    };

    var injectedEffect = function injectedEffect() {
      injectedCleanup();
      var current = injectedEffect.current;

      if (current) {
        injectedCleanup.cleanup = current();
      }
    };

    injectedEffect.current = rawEffect;
    currentInstance._effectStore[id] = {
      effect: injectedEffect,
      cleanup: injectedCleanup,
      deps: deps
    };
    injectEffect('componentDidMount', injectedEffect);
    injectEffect('componentWillUnmount', injectedCleanup);

    if (!deps) {
      injectEffect('componentDidUpdate', injectedEffect);
      injectEffect('componentWillUpdate', injectedCleanup);
    }
  } else {
    var _currentInstance$_eff = currentInstance._effectStore[id],
        effect = _currentInstance$_eff.effect,
        _currentInstance$_eff2 = _currentInstance$_eff.deps,
        prevDeps = _currentInstance$_eff2 === void 0 ? [] : _currentInstance$_eff2;

    if (!deps || deps.some(function (d, i) {
      return d !== prevDeps[i];
    })) {
      effect.current = rawEffect;
    } else {
      effect.current = null;
    }
  }
}

function injectEffect(key, fn) {
  currentInstance[key] = fn;
}

function withHooks(_render) {
  var WithHooks =
  /*#__PURE__*/
  function (_React$Component) {
    _inherits(WithHooks, _React$Component);

    function WithHooks() {
      var _this;

      _classCallCheck(this, WithHooks);

      _this = _possibleConstructorReturn(this, _getPrototypeOf(WithHooks).call(this));
      _this.state = {};
      _this._effectStore = [];
      currentInstance = _assertThisInitialized(_assertThisInitialized(_this));
      isMounting = true;
      _this.ret = _render(_this.props);
      return _this;
    }

    _createClass(WithHooks, [{
      key: "render",
      value: function render() {
        if (isMounting) {
          isMounting = false;
          callIndex = 0;
          return this.ret;
        }

        currentInstance = this;

        var ret = _render(this.props);

        currentInstance = null;
        callIndex = 0;
        return ret;
      }
    }]);

    return WithHooks;
  }(_react.default.Component);

  WithHooks.displayName = _render.name;
  return WithHooks;
}